package hr.genetics.models;
import hr.geometrics.Vector;

import java.util.ArrayList;
import java.util.Arrays;
import java.util.List;
import java.util.Random;

public abstract class GenericVechileModel {
	
	public enum Action {LEFT, RIGHT, CONTINUE};
	
	private int[] dimensions;
	protected Action[] DNA;
	protected int DNASize;
	
	protected List<Sensor> sensors;
	
	private double radius = 2;
	
	protected Vector currentSpeed = null;
	
	protected SimulationModel reality;
	
	public GenericVechileModel() {
		
		
	}
	
	public GenericVechileModel(int[] dimensions) {
		
		this.dimensions = new int[dimensions.length];
		
		int size = 1;
		for (int i = dimensions.length-1; i >= 0; --i) {
			this.dimensions[i] = size;
			size *= dimensions[i];
		}
		
		this.setDNASize(size);
				
		this.sensors = new ArrayList<Sensor>();
		
		this.generateRandomChromosomes();
		this.initialiseSensors();
				
	}
	
	public abstract void initialiseSensors();
	
	public abstract void checkSensors();
	
	protected abstract void calculateNewSpeed();
		
	private void generateRandomChromosomes() {

		Random pick = new Random();
		int random;
		int actionsTotal = Action.values().length;
		List<Action> values = Arrays.asList(Action.values());
				
		for (int i = 0; i < this.DNA.length; ++i) {
			random = pick.nextInt(actionsTotal);
			this.DNA[i] = values.get(random);
		}
		
	}
	
	public List<Sensor> getSensors() {
		
		return this.sensors;
		
	}
	
	public Action getChromosome(int[] positions) {
		
		int position = 0;
		for (int i = 0; i < positions.length; ++i) {
			if (i >= this.dimensions.length) break;
			position += this.dimensions[i] * positions[i];
		}
		
		return this.DNA[position];
		
	}
	
	public List<Double> getSensorsInfo() {
		
		List<Double> sensorsInfo = new ArrayList<Double>();
		
		List<Sensor> sensors = this.getSensors();
		for (int i = 0; i < sensors.size(); ++i) {
			sensorsInfo.add(sensors.get(i).getValue());
		}
		
		return sensorsInfo;
		
	}
	
	/**
	 * @return the radius
	 */
	public double getRadius() {
		return radius;
	}
	
	/**
	 * @return the dNA
	 */
	public Action[] getDNA() {
		return DNA;
	}
	
	/**
	 * @param dNA the dNA to set
	 */
	public void setDNA(Action[] dNA) {
		DNA = dNA;
	}

	/**
	 * @return the dimensions
	 */
	public int[] getDimensions() {
		return dimensions;
	}
	
	/**
	 * @param dimensions the dimensions to set
	 */
	public void setDimensions(int[] dimensions) {
		this.dimensions = dimensions;
	}

	/**
	 * @return the dNASize
	 */
	public int getDNASize() {
		return DNASize;
	}

	/**
	 * @param dNASize the dNASize to set
	 */
	public void setDNASize(int DNASize) {
		this.DNASize = DNASize;
		
		this.DNA = new Action[this.getDNASize()];
	}
	
	public Vector getSpeed() {
		return this.currentSpeed;
	}

	protected void setSpeed(Vector v) {
		this.currentSpeed = v;
	}
	
	public void setReality(SimulationModel s) {
		this.reality = s;
	}
	
}
