package hr.genetics.models;

import hr.geometrics.Line;

import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.ObjectInputStream;
import java.io.ObjectOutputStream;
import java.io.Serializable;
import java.nio.file.Paths;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

public class Path implements Serializable {

	private static final long serialVersionUID = -4778654988693886196L;
	
	private static Map<Integer, Path> paths;
	
	private String name;
	private int id;
	private int width = 600;
	private int height = 600;
	
	private List<Line> lines;
	private List<Checkpoint> checkpoints;
	private Line startLine;
	
	public Path(String name) {
		
		this.name = name;
		this.lines = new ArrayList<Line>();
		this.checkpoints = new ArrayList<Checkpoint>();
		
	}
	
	public String getName() {
		
		if (name == null || name.equals("Staza"))
			return "Staza " + new Integer(id).toString();
		
		return this.name;
		
	}
	
	public void setName(String name) {
		this.name = name;
	}
	
	public void addLine(Line l) {
		
		this.lines.add(l);
		
	}
	
	public Line getLine(int index) {
		
		return this.lines.get(index);
		
	}
	
	public List<Line> getLines() {
		
		return this.lines;
		
	}
	
	public void addCheckpoint(Checkpoint c) {
		
		this.checkpoints.add(c);
		
	}
	
	public Checkpoint getCheckpoint(int index) {
		
		return this.checkpoints.get(index);
		
	}
	
	public List<Checkpoint> getCheckpoints() {
		
		return this.checkpoints;
		
	}
	
	public void setStartLine(Line s) {
		
		this.startLine = s;
		
	}
	
	public Line getStartLine() {
		
		return this.startLine;
		
	}

	/**
	 * @return the width
	 */
	public int getWidth() {
		return width;
	}

	/**
	 * @param width the width to set
	 */
	public void setWidth(int width) {
		this.width = width;
	}

	/**
	 * @return the height
	 */
	public int getHeight() {
		return height;
	}

	/**
	 * @param height the height to set
	 */
	public void setHeight(int height) {
		this.height = height;
	}

	/**
	 * @return the id
	 */
	public int getId() {
		return id;
	}

	/**
	 * @param id the id to set
	 */
	public void setId(int id) {
		this.id = id;
	}
	
	public static Map<Integer, Path> getPaths() {
		
		if (paths == null)
			loadPaths();
		
		return paths;
		
	}
	
	@SuppressWarnings("unchecked")
	public static void loadPaths() {
		
		FileInputStream fis = null;
		ObjectInputStream in = null;
		try {
			fis = new FileInputStream(Paths.get("/Users/mario/workspace/Seminar/data/paths").toFile());
			in = new ObjectInputStream(fis);
			paths = (Map<Integer, Path>) in.readObject();
			in.close();
		} catch (FileNotFoundException e) {
			//e.printStackTrace();
		} catch(IOException ex) {
			ex.printStackTrace();
		} catch(ClassNotFoundException ex) {
			ex.printStackTrace();
		} 
				
		if (paths == null)
			paths = new HashMap<>();
				
	}
	
	public static void addPath(Path newPath) {
		
		Map<Integer, Path> paths = getPaths();
		
		if (newPath != null) {
			paths.put(newPath.getId(), newPath);
			serializePaths();
		}
		
	}
	
	public static boolean removePath(Integer pathID) {
		
		if (paths.containsKey(pathID)) {
			paths.remove(pathID);
			serializePaths();
			return true;
		}
		
		return false;
		
	}
	
	public static void serializePaths() {
		
		FileOutputStream fos = null;
		ObjectOutputStream out = null;
		try {
			fos = new FileOutputStream(Paths.get("/Users/mario/workspace/Seminar/data/paths").toFile());
			out = new ObjectOutputStream(fos);
			out.writeObject(getPaths());
			out.close();    
		} catch(IOException ex) {
			ex.printStackTrace();
		}	
		
	}
	
}
